@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Vocabulary;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Base role for JSON Schema vocabulary classes
 
-our $VERSION = '0.622';
+our $VERSION = '0.623';
 
 use 5.020;
 use Moo::Role;
@@ -80,6 +80,9 @@ sub eval_subschema_at_uri ($class, $data, $schema, $state, $uri) {
   abort($state, 'EXCEPTION: bad reference to "%s": not a schema', $schema_info->{canonical_uri})
     if $schema_info->{document}->get_entity_at_location($schema_info->{document_path}) ne 'schema';
 
+  my $scope_uri = $schema_info->{canonical_uri}->clone->fragment(undef);
+  push $state->{dynamic_scope}->@*, $scope_uri if $state->{dynamic_scope}->[-1] ne $scope_uri;
+
   return $state->{evaluator}->_eval_subschema($data, $schema_info->{schema},
     +{
       %$state,
@@ -105,7 +108,7 @@ JSON::Schema::Modern::Vocabulary - Base role for JSON Schema vocabulary classes
 
 =head1 VERSION
 
-version 0.622
+version 0.623
 
 =head1 SYNOPSIS
 
